/*global window, rJS, jIO, FormData */
/*jslint indent: 2, maxerr: 3 */
(function (window, rJS, jIO) {
  "use strict";

  function waitForServiceWorkerActive(registration) {
    var serviceWorker;
    if (registration.installing) {
      serviceWorker = registration.installing;
    } else if (registration.waiting) {
      serviceWorker = registration.waiting;
    } else if (registration.active) {
      serviceWorker = registration.active;
    }
    if (serviceWorker.state !== "activated") {
      return RSVP.Promise(function (resolve, reject) {
        serviceWorker.addEventListener('statechange', function (e) {
          if (e.target.state === "activated") {
            resolve();
          }
        });
        RSVP.delay(500).then(function () {
          reject(new Error("Timeout service worker install"));
        });
      });
    }
  }

  rJS(window)

    .ready(function (gadget) {
      // Initialize the gadget local parameters
      gadget.state_parameter_dict = {};
      gadget.state_parameter_dict.default_document = window.location.protocol +
        "//" + window.location.host +
        window.location.pathname.substring(
          0,
          window.location.pathname.length - "gadget_jio_crib.html".length
        ) +   "/";
      console.log(gadget.state_parameter_dict.default_document);
      this.state_parameter_dict.jio_storage = jIO.createJIO({
        "type": "indexeddb",
        "database": "ojs_source_code"
      });
      return this.state_parameter_dict.jio_storage.put(gadget.state_parameter_dict.default_document, {})
    })
    .ready(function (gadget) {
      if ('serviceWorker' in navigator) {
        // XXX Hack to not add a new service worker when one is already declared
        if (!navigator.serviceWorker.controller) {
          return new RSVP.Queue()
            .push(function () {
              return navigator.serviceWorker.register(
                '/gadget_cribjs_bootloader_serviceworker.js',
                {scope: '/'}
              );
            })
            .push(function (registration) {
              return waitForServiceWorkerActive(registration);
            })
        }
      } else {
        throw "Service Worker are not available in your browser";
      }
    })

    .declareMethod('get', function (url) {
      var storage = this.state_parameter_dict.jio_storage;
      return storage.getAttachment(this.state_parameter_dict.default_document, url)
        .push(function (result) {
          return jIO.util.readBlobAsDataURL(result);
        })
        .push(function (e) {
          return e.target.result;
        });
    })
    .declareMethod('put', function (url, data_uri) {
      var storage = this.state_parameter_dict.jio_storage;
      data_uri = jIO.util.dataURItoBlob(data_uri);
      console.log(this.state_parameter_dict.default_document);
      console.log(url);
      return storage.putAttachment(
        this.state_parameter_dict.default_document,
        url,
        data_uri
      );
    })
    .declareMethod('allDocs', function () {
      var storage = this.state_parameter_dict.jio_storage;
      console.log(this.state_parameter_dict.default_document);
      return storage.allAttachments(this.state_parameter_dict.default_document)
        .push(function(result) {
          console.log(result);
          return result;
      })
    })
    .declareMethod('remove', function (url) {
      var storage = this.state_parameter_dict.jio_storage;
      return storage.removeAttachment(
        this.state_parameter_dict.default_document,
        url
      );
    });

}(window, rJS, jIO));