/*jslint browser: true, indent: 2, maxlen: 80*/
/*globals RSVP, rJS,
         location, console, fetch, Promise*/
(function (window, document, RSVP, rJS, location, console, JSZip) {
  "use strict";

  function getExtension(url) {
    var extension = url.split('.').pop();
    if (extension.endsWith('/')) {
      return ".html";
    }
    return "." + extension;
  }

  function getSetting(gadget, key, default_value) {
    if (key === "site_editor_gadget_url") {
      return window.location.protocol + "//" + window.location.host +
        "/crib-enable.html";
    }
    return default_value;
  }

  function loadZipIntoCrib(crib_sw_gadget, zip, from_path, path_to_load) {
    var promise_list = [], url_number = 0,
      site_url = window.location.protocol + "//" + window.location.host;
    zip.forEach(function (relativePath, zipEntry) {
      var end_url;
      url_number += 1;
      if (zipEntry.dir) {
        return;
      }
      if (!relativePath.startsWith(from_path)) {
        return;
      }
      relativePath = relativePath.substring(from_path.length);
      console.log(relativePath);
      if (relativePath.startsWith("/")) {
        end_url = relativePath.substring(1);
      } else {
        end_url = relativePath;
      }
      promise_list.push(
        new RSVP.Queue()
        .push(function () {
          return zipEntry.async('blob');
        })
        .push(function (result) {
          if (end_url.endsWith(".js")) {
            // This is a ugly hack as mimetype needs to be correct for JS
            result = result.slice(0, result.size, "application/javascript");
          } else if (end_url.endsWith(".html")) {
            // This is a ugly hack as mimetype needs to be correct for JS
            result = result.slice(0, result.size, "text/html");
          } else if (end_url.endsWith(".css")) {
            // This is a ugly hack as mimetype needs to be correct for JS
            result = result.slice(0, result.size, "text/css");
          }
          return crib_sw_gadget.put(end_url, {blob: result});
        })
      );
    });
    return RSVP.all(promise_list);
  }

  function loadContentFromZIPFile(gadget, options) {
    var path_to_load = options.to_path,
      from_path = options.from_path,
      file_list = options.file_list;
    if (file_list.length === 0) {
      return "No File to Load";
    }
    return new RSVP.Queue()
    .push(function () {
      return RSVP.all([
        gadget.getDeclaredGadget('crib_sw_gadget'),
        JSZip.loadAsync(file_list[0])
      ]);
    })
    .push(function (result_list) {
      return loadZipIntoCrib(
        result_list[0], result_list[1], from_path, path_to_load
      );
    })
    .push(console.log, console.log);
  }


  function loadContentFromZIPURL(gadget, options) {
    var path_to_load = options.to_path, file_list, crib_sw_gadget,
      from_path = options.from_path, zip_url = options.zip_url,
      jio_gadget, url_list = [], url_number = 0;
    return new RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget('crib_sw_gadget');
    })
    .push(function (returned_gadget) {
      crib_sw_gadget = returned_gadget;
      return fetch(zip_url)
        .then(function (response) {                       // 2) filter on 200 OK
            if (response.status === 200 || response.status === 0) {
              return Promise.resolve(response.blob());
            } else {
              return Promise.reject(new Error(response.statusText));
            }
          });
    })
    .push(JSZip.loadAsync)
    .push(function (zip) {
      return loadZipIntoCrib(crib_sw_gadget, zip, from_path, path_to_load);
    })
    .push(console.log, console.log);
  }

  rJS(window)
    .ready(function (g) {
    g.props = {};
  })
    .declareMethod('loadFromZipUrl', function (options) {
      return loadContentFromZIPURL(this, options);
    })
    .declareMethod('loadFromZipFile', function (options) {
      return loadContentFromZIPFile(this, options);
    })
    .allowPublicAcquisition("getSetting", function (argument_list) {
      return getSetting(this, argument_list[0], argument_list[1]);
    });

}(window, document, RSVP, rJS, location, console, JSZip));